import sys
from anubis.results_utils.results_utils_arguments import parse_arguments
from anubis import results


def main():
    args = parse_arguments()
    results.create_aggregate(args.files, args.output_file)


if __name__ == '__main__':
    main()
    sys.exit(0)



from .arg_parser import parse_arguments
import os
from subprocess import call


def command_generator(account_feature_groups: list) -> str:
    """
    Use args, accounts, and features to construct behave command
    :param account_feature_groups:
    :return:
    """
    # get arguments
    known_args, unknown_args = parse_arguments()

    if type(account_feature_groups[0]) is int:
        process_name = account_feature_groups[0]
    else:
        # get data for constructing behave command (this is messy, but it works)
        process_name = account_feature_groups[0][0]

    # construct the behave command
    results_json_file = os.path.join(known_args.output, f'{process_name}.json')
    defined_args_formatted = ' '.join('-D {}'.format(arg[0]) for arg in known_args.D) if known_args.D else ''
    unknown_args_formatted = ' '.join(arg for arg in unknown_args)
    tags_formatted = ' '.join('--tags="@{}" '.format(t[0].replace("@", "")) for t in known_args.tags)
    scenarios = ' '.join(scenario for scenario in account_feature_groups[1])
    output_file = f'-f json -o "output/{process_name}.json"'

    command = f'behave {defined_args_formatted} {unknown_args_formatted} {tags_formatted} {output_file} {scenarios}'

    print(command, end='\n\n')
    r = call(command, shell=True)
    return results_json_file



import json
from pathlib import Path
from datetime import datetime
import argparse
from sys import exit


def _parse_arguments():
    parser = argparse.ArgumentParser('JSON to junit')
    parser.add_argument('--json_path', required=True)
    parser.add_argument('--junit_path', required=True)
    return parser.parse_args()


def _convert_json_to_junit(json_path: Path, junit_path: Path):
    with open(json_path) as f:
        json_results = json.load(f)

    statuses = [el['status'] for feature in json_results for el in feature['elements'] if el['type'] != 'background']
    passed = statuses.count('passed')
    failed = statuses.count('failed')
    total = passed + failed

    with open(junit_path, 'w+') as f:
        now = datetime.now()

        element_data = []
        total_time = 0
        for feature in json_results:
            for element in feature['elements']:
                element_junit_string = ''
                if element['type'] != 'background':
                    el_name = element['name'].replace('"', '&quot;')
                    el_status = element['status']
                    el_duration = 0
                    el_steps = ''
                    el_error = ''
                    el_tags = ' '.join(['@' + tag for tag in element['tags']])

                    for step in element['steps']:
                        el_duration += step['result']['duration'] if 'result' in step else 0
                        el_steps += '\t\t' + step['keyword'] + ' ' + step['name'] + '\n'

                        # print(step)
                        if 'result' in step and 'status' in step['result'] and step['result']['status'] == 'failed':
                            el_error += f'\t\t<error type="XXX">\n'
                            el_error += "".join(step["result"]["error_message"])\
                                .replace('&', '&amp;')\
                                .replace('<', '&lt;')\
                                .replace('>', '&gt;')\
                                .replace('”', '&quot;')\
                                .replace('‘', '&apos;')

                            el_error += '\n \t\t</error>\n'


                    system_out = (
                        f'\t\t<system-out>\n'
                        '\t\t\t<![CDATA[\n@scenario.begin\n\n' + '\t' + el_tags + '\n\t' + f'Scenario: {el_name}\n'
                        + el_steps + '\n\n@scenario.end\n'
                        '--------------------------------------------------------------------------------\n]]>\n'
                        '\t\t</system-out>\n'
                    )

                    element_junit_string += f'\t<testcase classname="{feature["name"]}" name="{el_name}" status="{el_status}" time="{el_duration}">\n'
                    element_junit_string += el_error
                    element_junit_string += system_out
                    element_junit_string += '\t</testcase>\n'

                    total_time += el_duration
                    element_data.append(element_junit_string)

        f.write(f'<testsuite name="Regression" tests="{total}" errors="{failed}" failures="{failed}" skipped="0" '
                f'time="{total_time}" timestamp="{now.isoformat()}" hostname="N/A">\n')
        for element in element_data:
            f.write(element)

        f.write('</testsuite>')


def main():
    args = _parse_arguments()
    _convert_json_to_junit(args.json_path, args.junit_path)


if __name__ == '__main__':
    # run everything
    main()
    exit(0)



import argparse


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument('--files', '-f', required=True, default=[], nargs='+')
    parser.add_argument('--output_file', '-o', required=True)
    return parser.parse_args()



import glob
import subprocess
from random import shuffle


def get_test_paths(unit, feature_dir, processes, accounts=None):
    paths = __get_paths(feature_dir, unit)
    result = __get_path_groups(paths, processes, accounts)
    return result


def __get_path_groups(paths, num_split, accounts=None, section=None):
    # split as evenly as possible
    inc = -(-len(paths) // num_split)

    # group the paths
    path_groups = [paths[i:i + inc] for i in range(0, len(paths), inc)]
    shuffle(path_groups)

    if accounts:
        return list(zip(accounts, path_groups))

    return list(zip(list(range(len(path_groups))), path_groups))


def __get_paths(feature_dir, unit):
    feature_paths = glob.glob(f'{feature_dir}/**/*.feature', recursive=True)

    if unit.lower() == 'scenario':
        scenario_paths = subprocess.run(
            [f'grep -irn "Scenario:" {" ".join(feature_paths)} | sed s/": .*"/""/'],
            shell=True,
            capture_output=True
        ).stdout.replace(b'\n', b' ').decode('utf-8').split()

        example_paths = subprocess.run(
            [f'grep -rin "|" {" ".join(feature_paths)} | sed s/": .*"/""/'],
            shell=True,
            capture_output=True
        ).stdout.replace(b'\n', b' ').decode('utf-8').split()

        return scenario_paths + example_paths

    return feature_paths



from selenium.webdriver.support.ui import WebDriverWait as WDW
from functools import wraps
import importlib
import time
from inspect import getfullargspec


def handle_locator(function):
    """
    This wraps a function that takes a locator, an arbitrary number of args, and an arbitrary number of kwargs.
    If the locator length is greater than 2, it assumes the tuple contains page object data and sets the page.
    If there are kwargs, it assumes they must be used to format the locator string and handles the interpolation.
    Finally, it runs the function.
    """
    @wraps(function)
    def wrapper(self, *args, **kwargs):
        locator = args[0]
        page_object_location = None

        # separate the locator from the new page
        if len(locator) == 3:
            page_object_location = locator[2]
            locator = locator[:2]

        # interpolate values into locator string if necessary
        locator = (locator[0], locator[1].format_map(kwargs))

        # perform the wrapped function
        function_call = function(self, locator, *args[1:], **kwargs)

        # if new page, update the `current_page` attribute of the the context object
        if '.click' in function.__repr__() and page_object_location:
            WDW(self.driver, 10).until(lambda _: self.driver.execute_script('return document.readyState') == 'complete')
            page_object = importlib.import_module(f'page_objects.{page_object_location}', 'Page')
            if hasattr(self.context, 'current_page'):
                self.context.current_page = getattr(page_object, 'Page')(self.context)
            elif hasattr(self.context, 'page'):
                self.context.page = getattr(page_object, 'Page')(self.context)
            time.sleep(3)   # this allows enough time for the page to load
        return function_call
    return wrapper


def wait_for_page_to_load(function):
    @wraps(function)
    def wrapper(*args, **kwargs):
        argspec = getfullargspec(function)
        driver = args[argspec.args.index('driver')]
        try:
            WDW(driver, 10).until(lambda _: driver.execute_script('return document.readyState') == 'complete')
        except Exception as e:
            print(e)

        return function(*args, **kwargs)
    return wrapper



import argparse


def parse_arguments():
    parser = argparse.ArgumentParser('Running in parallel mode')
    parser.add_argument('--processes', required=True, default=1, type=int)
    parser.add_argument('--feature_dir', required=True, default='features')
    parser.add_argument('--output', required=True, default='output')
    parser.add_argument('--aggregate', required=True, default='output/.tempres.json')
    parser.add_argument('--unit', required=False, default='scenario')
    parser.add_argument('--tags', required=False, nargs='*', action='append')
    parser.add_argument('-D', required=False, nargs='*', action='append')
    return parser.parse_known_args()



# selenium dependencies
from selenium.webdriver.remote.webelement import WebElement
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait as WDW
from selenium.webdriver.support import expected_conditions as EC

# custom dependencies
from anubis.page_object_utils.decorators import handle_locator
from anubis.page_object_utils import _shared_driver_functions


class BasePage:
    def __init__(self, context):
        self.context = context
        self.driver: webdriver = context.driver
        self.url = context.env_data['base_url']

    # <editor-fold> desc="Simple browser controls ...">
    def nav_to_url(self, url: str) -> None:
        self.driver.get(url)

    def refresh(self) -> None:
        self.driver.refresh()

    def go_back(self) -> None:
        self.driver.back()

    def go_forward(self) -> None:
        self.driver.forward()

    def scroll_top(self) -> None:
        self.driver.execute_script('window.scrollTo(0, 0)')

    def scroll_bottom(self) -> None:
        self.driver.execute_script('window.scrollTo(0, document.body.scrollHeight)')

    def shut_down(self) -> None:
        self.driver.close()
        self.driver.quit()

    def switch_to_tab_by_position(self, position: int) -> None:
        self.driver.switch_to.window(self.driver.window_handles[position])

    def close_tab_by_position(self, position: int) -> None:
        raise NotImplementedError
    # </editor-fold>

    # <editor-fold> desc="Getters ...">
    @handle_locator
    def get_element(self, by_locator: tuple, **kwargs) -> WebElement:
        return _shared_driver_functions.get_element(self.driver, by_locator, **kwargs)

    @handle_locator
    def get_elements(self, by_locator: tuple, **kwargs) -> list:
        return _shared_driver_functions.get_elements(self.driver, by_locator, **kwargs)

    def get_current_url(self) -> str:
        return self.driver.current_url

    def get_current_title(self) -> str:
        return self.driver.title

    @handle_locator
    def get_element_attribute(self, by_locator: tuple, attribute: str, **kwargs) -> str:
        return _shared_driver_functions.get_element_attribute(self.driver, by_locator, attribute, **kwargs)

    @handle_locator
    def get_element_text(self, by_locator: tuple, **kwargs) -> str:
        return _shared_driver_functions.get_element_text(self.driver, by_locator, **kwargs)
    # </editor-fold>

    # <editor-fold> desc="Visibility ...">
    @handle_locator # todo - check
    def is_visible(self, by_locator: tuple, **kwargs) -> bool:
        return _shared_driver_functions.is_visible(self.driver, by_locator, **kwargs)

    @handle_locator # todo - check
    def wait_until_visible(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.wait_until_visible(by_locator, **kwargs)

    @handle_locator # todo - check
    def wait_until_invisible(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.wait_until_invisible(by_locator, **kwargs)

    def wait_for_page_to_load(self) -> None:
        try:
            WDW(self.driver, 10).until(lambda _: self.driver.execute_script('return document.readyState') == 'complete')
        except Exception as e:
            print(e)
    # </editor-fold>

    # <editor-fold desc="Element Interactions ...">
    @handle_locator
    def drag_element_by_offset(self, by_locator: tuple, x_offset: int, y_offset: int, **kwargs) -> None:
        return _shared_driver_functions.drag_element_by_offset(self.driver, by_locator, x_offset, y_offset, **kwargs)

    @handle_locator
    def click_element_by_offset(self, by_locator: tuple, x_offset: int, y_offset: int, **kwargs) -> None:
        return _shared_driver_functions.click_element_by_offset(self.driver, by_locator, x_offset, y_offset, **kwargs)

    @handle_locator
    def click(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.click(self.driver, by_locator, **kwargs)

    @handle_locator
    def enter_text(self, by_locator: tuple, text: str, **kwargs) -> None:
        return _shared_driver_functions.enter_text(self.driver, by_locator, text, **kwargs)

    @handle_locator
    def hover_to(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.hover_to(self.driver, by_locator, **kwargs)

    @handle_locator
    def highlight_element(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.highlight_element(self.driver, by_locator, **kwargs)
    # </editor-fold>


class BaseModal:
    def __init__(self, context):
        self.context = context
        self.driver = context.driver
        self.locators = None

    @handle_locator
    def click(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.click(self.driver, by_locator, **kwargs)

    def close(self):
        if hasattr(self.locators, 'close_button'):
            self.click(self.locators.close_button)
        else:
            print('Could not find "close_button" locator')

    def confirm(self):
        if hasattr(self.locators, 'confirm'):
            self.click(self.locators.confirm)
        elif hasattr(self.locators, 'okay'):
            self.click(self.locators.okay)
        else:
            print('Could not find okay/confirm locator')

    @handle_locator
    def get_element(self, by_locator: tuple, **kwargs) -> WebElement:
        return _shared_driver_functions.get_element(self.driver, by_locator, **kwargs)

    @handle_locator
    def get_elements(self, by_locator: tuple, **kwargs) -> list:
        return _shared_driver_functions.get_elements(self.driver, by_locator, **kwargs)

    @handle_locator
    def enter_text(self, by_locator: tuple, text: str, **kwargs) -> None:
        return _shared_driver_functions.enter_text(self.driver, by_locator, text, **kwargs)

    @handle_locator
    def hover_to(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.hover_to(self.driver, by_locator, **kwargs)

    def modal_is_displayed(self):
        try:
            element = WDW(self.driver, 3).until(EC.visibility_of_element_located(self.locators.modal))
            return True
        except:
            return False

    @handle_locator
    def wait_until_visible(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.wait_until_visible(by_locator, **kwargs)

    @handle_locator
    def wait_until_invisible(self, by_locator: tuple, **kwargs) -> None:
        return _shared_driver_functions.wait_until_invisible(by_locator, **kwargs)

    @handle_locator
    def click_element_by_offset(self, by_locator: tuple, x_offset: int, y_offset: int, **kwargs) -> None:
        return _shared_driver_functions.click_element_by_offset(self.driver, by_locator, x_offset, y_offset, **kwargs)


class BaseLocator:
    def __add__(self, other):
        combination = {}
        combination.update(self.__dict__)
        combination.update(other.__dict__)
        new = self.__class__()
        new.__dict__.update(combination)
        return new

    def __iadd__(self, other):
        combination = {}
        combination.update(self.__dict__)
        combination.update(other.__dict__)
        new = self.__class__()
        new.__dict__.update(combination)
        return new



import argparse
import csv
import json
from datetime import datetime


def cucumber_json_to_csv(json_input_file_path: str, csv_output_file_path) -> None:
    # open the json and store as dictionary
    with open(json_input_file_path, 'r') as f:
        results = json.load(f)

    # open and create a csv file
    with open(csv_output_file_path, 'w', newline='') as csvfile:
        test = csv.writer(csvfile)
        test.writerow(['run_time', 'team', 'feature_name', 'scenario_name', 'scenario_tags', 'scenario_status', 'error_message', 'duration', 'location'])

        # get data from the file name
        try:
            time_stamp = datetime.fromisoformat(json_input_file_path.split('__')[2].replace('Z.json', ''))  # todo - make this better
            team = json_input_file_path.split('__')[1]
        except IndexError as e:
            print(f'There was an error getting the time stamp or team\n{e}')
            time_stamp = datetime.now()
            team = None

        # for every scenario, write important data to a row in the csv
        for feature in results:
            feature_name = feature['name']
            for element in feature['elements']:
                if element['type'].lower() == 'scenario':
                    scenario_duration = 0
                    error_message = ''

                    for step in element['steps']:
                        if 'result' in step:
                            scenario_duration += step['result']['duration']
                            if 'error_message' in step['result']:
                                error_message = '\n'.join(step['result']['error_message'])

                    row_data = [time_stamp, team, feature_name, element['name'], ' '.join(element['tags']), element['status'], error_message, scenario_duration, element['location']]
                    test.writerow(row_data)
                    # print(row_data) # debug


def parse_args():
    parser = argparse.ArgumentParser("Creating CSV")
    parser.add_argument('--json_input_file_path', '-j', required=True)
    parser.add_argument('--csv_output_file_path', '-c', required=True)
    return parser.parse_args()


def main():
    args = parse_args()
    cucumber_json_to_csv(args.json_input_file_path, args.csv_output_file_path)


if __name__ == '__main__':
    main()



from behave import fixture
from selenium.webdriver import Chrome, ChromeOptions, Safari, FirefoxOptions
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
import logging

# drivers
from selenium import webdriver
from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager

super_logger = logging.getLogger('ROOT')


@fixture
def chrome(context):
    # setup chrome options
    options = ChromeOptions()
    options.add_argument('--ignore-certificate-errors')

    if context.config.userdata['headless'].lower() == 'true':
        options.add_argument('--headless')
        options.add_argument('--ignore-certificate-errors')
        options.add_argument('--no-sandbox')
        options.add_argument('--verbose')
        options.add_argument('--disable-dev-shm-usage')
        options.add_argument('--window-size=1920,1080')

    caps = DesiredCapabilities.CHROME
    caps['goog:loggingPrefs'] = {'performance': 'ALL'}

    context.driver = webdriver.Chrome(ChromeDriverManager().install(), chrome_options=options, desired_capabilities=caps)
    # add auth cookie to browser
    # context.driver.add_cookie({
    #     'domain': context.env_data['domain'],
    #     'name':   context.env_data['cookie_name'],
    #     'value':  context.env_data['token']
    # })
    if hasattr(context, 'composite_request'):
        context.driver.get(context.composite_request.url)
    else:
        context.driver.get(context.env_data['base_url'])

    context.driver.maximize_window()
    yield context.driver
    context.driver.quit()


@fixture
def safari(context):
    context.driver = Safari()
    yield context.driver
    context.driver.quit()


@fixture
def firefox(context):
    ff_options = FirefoxOptions()
    if context.config.userdata['headless'].lower() == 'true':
        ff_options.headless = True
    context.driver = webdriver.Firefox(executable_path=GeckoDriverManager().install(), options=ff_options, service_log_path='/dev/null')  # todo - hard-coded `service_log_path` = probably problematic
    context.driver.maximize_window()
    context.driver.get(context.composite_request.url)
    yield context.driver
    context.driver.quit()



# __main__.py
import os
import json
import sys
import re
import multiprocessing
from datetime import datetime
from pathlib import Path
from types import SimpleNamespace

# custom
from anubis import account_splitter
from anubis import feature_splitter
from anubis import arg_parser
from anubis import results
from anubis.parallelizer import command_generator

ANUBIS_ASCII = ("""
                 ♡♡                                               
                ♡♡♡                                                 
              ♡♡ ♡♡♡♡                                               
         ♡♡♡♡♡♡♡♡♡♡♡♡                                               
                 ♡♡♡♡♡                                              
                  ♡♡♡♡♡                                             
               ♡♡♡♡♡♡♡♡♡                                            
              ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡        ♡♡♡♡♡♡                         
              ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡ ♡♡♡♡                     
              ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡                   
          ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡     ♡♡♡♡♡♡♡♡♡♡                  
♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡       ♡♡♡♡♡     ♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡♡                 
                                                    ♡♡♡              
          POWERED BY ANUBIS                          ♡♡♡♡            
(and the power of love  Σ>―(〃°ω°〃)♡→)               ♡♡♡♡♡          
                                                      ♡♡♡♡♡         
                                                       ♡♡♡♡         
                                                        ♡♡     """)


def main():
    start = datetime.now()
    print(ANUBIS_ASCII)

    # <editor-fold desc="--- Parse arguments">
    args_known, args_unknown = arg_parser.parse_arguments()
    user_definitions = SimpleNamespace()
    for arg in args_known.D:
        setattr(user_definitions, arg[0].split('=')[0], arg[0].split('=')[1])
    # </editor-fold>

    # <editor-fold desc="--- Set up the output directories for the run">
    # create a dir that will contain results and be exported
    print('--- SETTING UP OUTPUT')
    if not os.path.isdir(args_known.output):
        print(f'\tThe run output should go to <{args_known.output}> but it doesn\'t exist (creating it now)')
        os.makedirs(args_known.output, exist_ok=True)
    else:
        print(f'\tSending the output of this run to <{args_known.output}>')

    if not Path(args_known.aggregate).parent.is_dir():
        print(f'\tThe directory to store <{args_known.aggregate}> doesn\'t exist (creating it now)')
        Path(args_known.aggregate).parent.mkdir(parents=True, exist_ok=True)
    # </editor-fold>

    # set up the multiple processes
    multiprocessing.set_start_method('fork')
    pool = multiprocessing.Pool(args_known.processes)

    # <editor-fold desc="--- Set up the accounts for the run">
    # get account data available for parallel runs
    # the `accounts_data` list looks like this --> [(<run_name>, "user pass"), [<list of feature files>]]
    if hasattr(user_definitions, 'account_file') and hasattr(user_definitions, 'account_section'):
        print('\n--- PARSING ACCOUNTS')
        print(f'\tfile:    <{user_definitions.account_file}>')
        print(f'\tsection: <{user_definitions.account_section}>')
        accounts_data = account_splitter.get_accounts(
            args_known.processes, user_definitions.account_file, user_definitions.account_section)
    elif hasattr(user_definitions, 'account_file') and hasattr(user_definitions, 'env'):
        print('\n--- PARSING ACCOUNTS')
        print(f'\tfile:    <{user_definitions.account_file}>')
        accounts_data = account_splitter.get_accounts(
            args_known.processes, user_definitions.account_file, user_definitions.env)
    else:
        print('\n--- ACCOUNTS NOT SPECIFIED')
        print('\tusing timestamps, instead of accounts, for output file names')
        # create dummy account data just for the purpose of naming the runs
        accounts_data = [(re.sub(r'[-:.]', '', datetime.now().isoformat()), 'NONE NONE')
                         for _ in range(args_known.processes)]
    # </editor-fold>

    # <editor-fold desc="--- Set up the features">
    # split the features and store as list
    print('\n--- GROUPING FEATURES & ACCOUNTS')
    print(f'\tfeature dir: <{args_known.feature_dir}>')
    print(f'\ttags: <{",".join([t[0] for t in args_known.tags]) if args_known.tags else "(none)"}>')

    # parallel
    feature_groups = feature_splitter.get_test_paths(
        args_known.unit, args_known.feature_dir, args_known.processes, accounts_data)

    # run all the processes and save the locations of the result files
    num_groups = len(feature_groups)
    print(f'\n--- RUNNING <{num_groups} PROCESS{"ES" * int(num_groups > 1)}>')
    result_files = pool.map(command_generator, feature_groups)
    # </editor-fold>

    # <editor-fold desc="--- Combine the results">
    res_string = None
    total = 0
    try:
        print('--- HANDLING RESULTS')
        # create the aggregate file and calculate pass/fail rate
        results.create_aggregate(files=result_files, aggregate_out_file=args_known.aggregate)

        with open(args_known.aggregate) as f:
            res = json.load(f)

        statuses = []
        for feature in res:
            if 'elements' in feature:
                for scenario in feature['elements']:
                    if scenario['type'] != 'background':
                        statuses.append(scenario['status'])

        passed = statuses.count('passed')
        failed = statuses.count('failed')
        total = passed + failed
        res_string = f'{passed / total * 100:.2f}%'
    except Exception as e:
        passed, failed = 0, 0
        if args_known.aggregate:
            print(f'There was an error combining results\n{e}')
        else:
            pass

    if args_known.aggregate.startswith('.temp'):
        os.remove(args_known.aggregate)
    # </editor-fold>

    end = datetime.now()

    # <editor-fold desc="extremely basic summary">
    print('\n===========♡𓃥♡ SUMMARY ♡𓃥♡===========')
    print(f'Env:         <{user_definitions.env}>')
    print(f'Results:     <{args_known.aggregate}>') if hasattr(args_known, 'aggregate') else None
    print(f'Total tests: <{total}>')
    print(f'Pass Rate:   <{res_string if res_string else "could not calculate"}>')
    print(f'Run Time:    <{(end - start)}>')
    print('=======================================')
    # </editor-fold>


if __name__ == '__main__':
    # run everything
    main()
    sys.exit(0)



from selenium.webdriver import ActionChains
from selenium.webdriver.support.ui import WebDriverWait as WDW
from selenium.webdriver.support import expected_conditions as EC

from anubis.page_object_utils.decorators import wait_for_page_to_load


# getters
def get_element(driver, by_locator: tuple, **kwargs):
    return driver.find_element(*by_locator)


def get_elements(driver, by_locator: tuple, **kwargs):
    return driver.find_elements(*by_locator)


def get_element_attribute(driver, by_locator: tuple, attribute: str, **kwargs) -> str:
    return driver.find_element(*by_locator).get_attribute(attribute.lower())


def get_element_text(driver, by_locator: tuple, **kwargs) -> str:
    return driver.find_element(*by_locator).text


# clicks
@wait_for_page_to_load
def click(driver, by_locator: tuple, **kwargs) -> None:
    # locate and move to the element
    element = WDW(driver, 3.14).until(EC.visibility_of_element_located(by_locator))

    scroll_height = driver.execute_script('return document.body.scrollHeight')
    window_size = driver.get_window_size()['height']

    if element.location['y'] > (scroll_height - .5 * window_size):
        driver.execute_script('window.scrollTo(0, document.body.scrollHeight)')
    elif element.location['y'] < (.5 * window_size):
        driver.execute_script('window.scrollTo(0, 0)')
    elif not element.is_displayed():
        driver.execute_script(
            f"window.scrollTo({element.location['x']}, {element.location['y'] - .5 * window_size});")

    # wait for element to be clickable, then click it
    WDW(driver, 3.14).until(EC.element_to_be_clickable(by_locator)).click()


def click_element_by_offset(driver, by_locator: tuple, x_off: int, y_off: int, **kwargs) -> None:
    element = get_element(driver, by_locator)
    ActionChains(driver).move_to_element_with_offset(element, x_off, y_off).click().perform()


def drag_element_by_offset(driver, by_locator: tuple, x_off: int, y_off: int, **kwargs) -> None:
    element = get_element(driver, by_locator)
    ActionChains(driver).click_and_hold(element).move_by_offset(x_off, y_off).perform()


def click_if_valid(driver, valid_item_map: dict, item_name: str, **kwargs) -> None:
    if item_name.lower() in valid_item_map:
        click(driver, valid_item_map[item_name.lower()])
    else:
        print(f'You tried to click an item called <{item_name}>')
        print(f'Make sure to select from the following:\n{list(valid_item_map.keys())}')


# visibility
@wait_for_page_to_load
def wait_until_visible(driver, by_locator: tuple, **kwargs):
    WDW(driver, 3.14).until(EC.visibility_of_element_located(by_locator))


@wait_for_page_to_load
def wait_until_invisible(driver, by_locator: tuple, **kwargs):
    WDW(driver, 3.14).until(EC.invisibility_of_element_located(by_locator))


@wait_for_page_to_load
def is_visible(driver, by_locator: tuple, **kwargs) -> bool:
    try:
        element = WDW(driver, 3.14).until(EC.visibility_of_element_located(by_locator))
        return element.is_displayed()
    except Exception as e:
        return False


# other
def enter_text(driver, by_locator: tuple, text, **kwargs):
    return WDW(driver, 3.14).until(EC.visibility_of_element_located(by_locator)).send_keys(text)


def hover_to(driver, by_locator: tuple, **kwargs) -> None:
        el = WDW(driver, 3.14).until(EC.visibility_of_element_located(by_locator))
        window_size = driver.get_window_size()['height']

        driver.execute_script(f"window.scrollTo({el.location['x']}, {el.location['y'] - .5 * window_size});")
        ActionChains(driver).move_to_element(el).perform()


def highlight_element(driver, by_locator: tuple, **kwargs) -> None:
    el = driver.find_element(*by_locator)
    driver.execute_script('arguments[0].setAttribute("style", "background: yellow; border: 2px solid red;");', el)
